// Environment imports
const Clutter = imports.gi.Clutter;
const ExtensionUtils = imports.misc.extensionUtils;
const Extension = ExtensionUtils.getCurrentExtension();
const Settings = Extension.imports.settings;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Lang = imports.lang;
const Mainloop = imports.mainloop;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const St = imports.gi.St;

// Application imports
const Main = imports.ui.main;
const Soup = imports.gi.Soup;
const Shell = imports.gi.Shell;

// Localization
const Gettext = imports.gettext;
const _ = Gettext.domain("status").gettext;

// Constants
const KEY_RETURN = 65293;
const KEY_ENTER  = 65421;

let _httpSession;

const StatusIndicator = new Lang.Class({
    Name: 'StatusIndicator',
    Extends: PanelMenu.Button,

    _init: function(metadata) {
        this.parent(0.0, "HTTP Status Indicator", false);

        // Locale
        this.meta = metadata;
        let locales = this.meta.path + "/locale";
        Gettext.bindtextdomain('status', locales);

        this._prefs = new Settings.Prefs();
        this._frequency = this._prefs.FREQUENCY.get();

        this._initFile();
        this._initButton();
        this._initMenu();

        this._refresh();
    },

    _initButton: function() {
        this.buttonText = new St.Label({
            text: _("Loading..."),
            y_align: Clutter.ActorAlign.CENTER
        });
        this.buttonIcon = new St.Icon({
            icon_name: 'network-transmit-receive-symbolic',
            style_class: 'system-status-icon'
        });

        let box = new St.BoxLayout();
        box.add_actor(this.buttonText);
        box.add_actor(this.buttonIcon);
        this.actor.add_child(box);
    },

    _initFile: function() {
        // Define file path
        this.filePath = GLib.get_home_dir() + "/.http-status-indicator.txt";

        // Check if file exists
        if (!GLib.file_test(this.filePath, GLib.FileTest.EXISTS)) {
            // Write empty file if necessary
            let file = Gio.file_new_for_path(this.filePath);
            let out = file.replace(null, false, Gio.FileCreateFlags.NONE, null);
            Shell.write_string_to_stream(out, '');
            out.close(null);
        }
    },

    _initMenu: function() {
        // Create input box
        let mainBox = new St.BoxLayout();
        mainBox.set_vertical(true);

        // Create urls box
        this.urlsBox = new St.BoxLayout();
        this.urlsBox.set_vertical(true);

        // Load urls
        this.reloadMenu();

        // Text entry
        this.newUrl = new St.Entry(
        {
            name: "newUrlEntry",
            hint_text: _("New URL..."),
            track_hover: true,
            can_focus: true
        });

        let entryNewUrl = this.newUrl.clutter_text;
        entryNewUrl.connect('key-press-event', Lang.bind(this, function(object, event) {
            let symbol = event.get_key_symbol();
            if (symbol == KEY_RETURN || symbol == KEY_ENTER) {
                this.addURL(object.get_text());
                entryNewUrl.set_text('');
            }
        }));

        // Bottom section
        var bottomSection = new PopupMenu.PopupMenuSection();
        bottomSection.actor.add_actor(this.newUrl);
        bottomSection.actor.add_style_class_name("newUrlEntry");
        mainBox.add_actor(bottomSection.actor);

        // Add elements on menu
        this.menu.box.add(this.urlsBox);
        this.menu.box.add(mainBox);
    },

    _refresh: function() {
        this._loadData(this._refreshUI);
        this._removeTimeout();
        this._timeout = Mainloop.timeout_add_seconds(this._frequency, Lang.bind(this, this._refresh));
        return true;
    },

    _loadData: function() {
        _httpSession = new Soup.Session();
        // Should respect standards https://tools.ietf.org/html/rfc7231#section-5.5.3
        _httpSession.user_agent = 'HTTP Status Monitor';
        this._refreshUI(_("Analyzing"), 'network-transmit-receive-symbolic');

        //let content = Shell.get_file_contents_utf8_sync(this.filePath);
        //let lines = content.toString().split('\n');
        
        let lines = this.menuItems;
        
        this.returnedData = 0;
        this.queryCount = lines.length;
        
        for (let i=0; i<lines.length; i++) {
            if (this._isURL(lines[i])) {
                let message = Soup.form_request_new_from_hash('HEAD', lines[i].linkLabel, {});
                _httpSession.queue_message(message, Lang.bind([this, lines[i]], function(_httpSession, message) {
                    this[0].returnedData++;
                    this[1].setStatus(message.status_code);
                    warningText = _("Warning");
                    if (message.status_code != 200) {
                        this[0]._refreshUI(warningText, 'network-error-symbolic');
                    } else {
                        if (this[0].returnedData >= this[0].queryCount - 1 && this[0].buttonText.get_text() != warningText) {
                            this[0]._refreshUI(_("Ok"), 'network-transmit-receive-symbolic');
                        }
                    }
                }));
            }
        }
    },

    _refreshUI: function(data, icon) {
        if (typeof icon !== "undefined") {
            this.buttonIcon.set_icon_name(icon);
        }
        this.buttonText.set_text(data);
    },

    _removeTimeout: function() {
        if (this._timeout) {
            Mainloop.source_remove(this._timeout);
            this._timeout = null;
        }
    },

    _isURL: function(url) {
        if (url === '' || url === '\n') {
            return false;
        }
        return true;
    },

    addHTTP: function(url) {
        if (!/^https?:\/\//i.test(url)) {
            url = "http://" + url;
        }
        return url;
    },

    addURL: function(text) {
        // Don't add empty URL
        if (!this._isURL(text)) {
            return;
        }

        // Normalize url
        text = this.addHTTP(text);

        // Don't add if URL already exists
        let lines = this.menuItems;
        for (let i=0; i<lines.length; i++) {
            if (lines[i].linkLabel == text) {
                return;
            }
        }

        // Write new text to file
        let content = Shell.get_file_contents_utf8_sync(this.filePath);
        content = content + text + "\n";
        let file = Gio.file_new_for_path(this.filePath);
        let out = file.replace(null, false, Gio.FileCreateFlags.NONE, null);
        Shell.write_string_to_stream(out, content);
        out.close(null);

        // Reload menu and check urls
        this.reloadMenu();
        this._refresh();
    },

    removeURL: function(url) {
        let content = Shell.get_file_contents_utf8_sync(this.filePath);
        content = content.replace(url + "\n", "");

        // Write new text to file
        let file = Gio.file_new_for_path(this.filePath);
        let out = file.replace(null, false, Gio.FileCreateFlags.NONE, null);
        Shell.write_string_to_stream(out, content);
        out.close(null);
    },

    reloadMenu: function() {
        this.urlsBox.destroy_all_children();
        this.menuItems = [];

        let content = Shell.get_file_contents_utf8_sync(this.filePath);
        let lines = content.toString().split('\n');
        for (let i=0; i<lines.length; i++) {
            if (this._isURL(lines[i])) {
                let item = new PopupUrlItem(lines[i], "???");
                this.menuItems.push(item);
                this.urlsBox.add(item.actor);
            }
        }
    },

    stop: function() {
        if (_httpSession !== undefined)
            _httpSession.abort();
        _httpSession = undefined;

        if (this._timeout)
            Mainloop.source_remove(this._timeout);
        this._timeout = undefined;

        this.menu.removeAll();
    }
});

const PopupUrlItem = new Lang.Class({
    Name: 'PopupUrlItem',
    Extends: PopupMenu.PopupMenuItem,

    _init: function(text, status) {
        this.parent(text);
        this.label.x_expand = true;

        this.linkLabel = text;
        this.statusLabel = new St.Label({
            text: _("???"),
            y_align: Clutter.ActorAlign.END
        });

        this.setStatus(status);

        this.removeButton = new St.Button({
            style_class: 'remove-button',
            x_align: 1,
            reactive: true,
            can_focus: true,
            track_hover: true,
            accessible_name: 'remove',
        });
        this.removeButton.child = new St.Icon({ icon_name: 'edit-delete-symbolic' });

        this.removeButton.connect('clicked', Lang.bind(this,function() {
            stMenu.removeURL(this.linkLabel);
            this.destroy();
        }));

        this.connect('activate', Lang.bind(this,function() {
            Gio.app_info_launch_default_for_uri(this.linkLabel, null);
            stMenu.menu.close();
        }));

        this.actor.add(this.statusLabel, { expand: false, x_align: St.Align.END });
        this.actor.add(this.removeButton, { expand: false, x_align: St.Align.END });
    },

    setStatus: function(code) {
        if (code < 100) {
            code = _("Down");
        }
        this.statusLabel.set_text(code.toString());
    }
});

let meta;
let stMenu;

function init(metadata) {
    meta = metadata;
}

function enable() {
    stMenu = new StatusIndicator(meta);
    Main.panel.addToStatusArea('st-indicator', stMenu);
}

function disable() {
    stMenu.stop();
    stMenu.destroy();
}
